@@ -602,6 +602,11 @@ char *flashbuses_to_text(enum chipbustype bustype)
 	return ret;
 }
 
+int probe_noop(struct flashctx *flash)
+{
+	return 1;
+}
+
 static int init_default_layout(struct flashctx *flash)
 {
 	/* Fill default layout covering the whole chip. */
@@ -828,11 +833,14 @@ static int check_block_eraser(const struct flashctx *flash, int k, int log)
 		return 1;
 	}
 
-	if (flash->chip->bustype == BUS_SPI) {
+	if (flash->chip->bustype == BUS_SPI && flash->chip->spi_cmd_set == SPI25) {
 		bool native_4ba;
 		int i;
 
 		const uint8_t *opcode = spi_get_opcode_from_erasefn(eraser.block_erase, &native_4ba);
+		if (!opcode)
+			return 1;
+
 		for (i = 0; opcode[i]; i++) {
 			if ((native_4ba && !spi_master_4ba(flash)) ||
 			    !flash->mst.spi->probe_opcode(flash, opcode[i])) {
@@ -1198,7 +1206,7 @@ static int walk_by_layout(struct flashctx *const flashctx, struct walk_info *con
 	int ret = 0, layout_count = 0;
 
 	all_skipped = true;
-	msg_cinfo("Erasing and writing flash chip... ");
+	msg_cinfo("Erasing %sflash chip... ", info->newcontents ? "and writing " : "");
 
 	if (do_erase) {
 		layout_count = create_erase_layout(flashctx, &erase_layouts);
@@ -1251,7 +1259,7 @@ static int walk_by_layout(struct flashctx *const flashctx, struct walk_info *con
 	}
 	if (all_skipped)
 		msg_cinfo("\nWarning: Chip content is identical to the requested image.\n");
-	msg_cinfo("Erase/write done.\n");
+	msg_cinfo("Erase%s done.\n", info->newcontents ? "/write" : "");
 
 free_ret:
 	free_erase_layout(erase_layouts, layout_count);
@@ -1533,11 +1541,11 @@ int selfcheck(void)
 	} else {
 		for (i = 0; i < flashchips_size - 1; i++) {
 			const struct flashchip *chip = &flashchips[i];
+			const char *const name = chip->name != NULL ? chip->name : "unnamed";
 			if (chip->vendor == NULL || chip->name == NULL || chip->bustype == BUS_NONE) {
 				ret = 1;
 				msg_gerr("ERROR: Some field of flash chip #%d (%s) is misconfigured.\n"
-					 "Please report a bug at flashprog@flashprog.org\n", i,
-					 chip->name == NULL ? "unnamed" : chip->name);
+					 "Please report a bug at flashprog@flashprog.org\n", i, name);
 			}
 			if (chip->feature_bits &
 			    (FEATURE_4BA_ENTER | FEATURE_4BA_ENTER_WREN | FEATURE_4BA_ENTER_EAR7 |
@@ -1545,8 +1553,23 @@ int selfcheck(void)
 			    && !chip->prepare_access) {
 				msg_gerr("ERROR: Flash chip #%d (%s) misses chip\n"
 					 "preparation function for 4BA and multi-i/o modes.\n"
-					 "Please report a bug at flashprog@flashprog.org\n", i,
-					 chip->name == NULL ? "unnamed" : chip->name);
+					 "Please report a bug at flashprog@flashprog.org\n", i, name);
+				ret = 1;
+			}
+			uint8_t zero_cycles[sizeof(chip->dummy_cycles)] = { 0 };
+			if ((chip->feature_bits & (FEATURE_QPI_35_F5 | FEATURE_QPI_38_FF)) &&
+			    !memcmp(&chip->dummy_cycles, zero_cycles, sizeof(zero_cycles))) {
+				msg_gerr("ERROR: Flash chip #%d (%s) misses QPI dummy-cycle\n"
+					 "settings. Please report a bug at flashprog@flashprog.org\n",
+					 i, name);
+				ret = 1;
+			}
+			if (chip->reg_bits.bp[0].reg != INVALID_REG &&
+			    (!chip->wp_write_cfg || !chip->wp_read_cfg ||
+			     !chip->wp_get_ranges || !chip->decode_range)) {
+				msg_gerr("ERROR: Flash chip #%d (%s) advertises block-protection\n"
+					 "bits, but misses one or more write-protection functions.\n"
+					 "Please report a bug at flashprog@flashprog.org\n", i, name);
 				ret = 1;
 			}
 			if (selfcheck_eraseblocks(chip)) {
@@ -1878,9 +1901,6 @@ int flashprog_image_write(struct flashctx *const flashctx, void *const buffer, c
 	if (verify && !all_skipped) {
 		msg_cinfo("Verifying flash... ");
 
-		/* Work around chips which need some time to calm down. */
-		programmer_delay(1000*1000);
-
 		if (verify_all)
 			combine_image_by_layout(flashctx, newcontents, oldcontents);
 		ret = verify_by_layout(flashctx, verify_layout, curcontents, newcontents);
