@@ -363,6 +363,14 @@ enum ich_chipset {
 	CHIPSET_APOLLO_LAKE,
 	CHIPSET_GEMINI_LAKE,
 	CHIPSET_ELKHART_LAKE,
+
+	CHIPSET_HAS_NEW_ACCESS_PERM, /****** BM_RAP/WAP regs from here on *****/
+
+	CHIPSET_C740_SERIES_EMMITSBURG = CHIPSET_HAS_NEW_ACCESS_PERM,
+	CHIPSET_SNOW_RIDGE,
+	CHIPSET_METEOR_LAKE,
+	CHIPSET_LUNAR_LAKE,
+	CHIPSET_ARROW_LAKE,
 };
 
 /* ichspi.c */
